@@ -16,9 +16,12 @@ use std::mem::transmute;
 use std::path;
 
 pub trait GstBinExtManual: 'static {
+    #[doc(alias = "gst_bin_add_many")]
     fn add_many<E: IsA<Element>>(&self, elements: &[&E]) -> Result<(), glib::BoolError>;
+    #[doc(alias = "gst_bin_remove_many")]
     fn remove_many<E: IsA<Element>>(&self, elements: &[&E]) -> Result<(), glib::BoolError>;
 
+    #[doc(alias = "do-latency")]
     fn connect_do_latency<F: Fn(&Self) -> Result<(), LoggableError> + Send + Sync + 'static>(
         &self,
         f: F,
@@ -43,16 +46,21 @@ pub trait GstBinExtManual: 'static {
     #[doc(alias = "get_children")]
     fn children(&self) -> Vec<Element>;
 
+    #[doc(alias = "gst_debug_bin_to_dot_data")]
     fn debug_to_dot_data(&self, details: crate::DebugGraphDetails) -> GString;
-    fn debug_to_dot_file<Q: AsRef<path::Path>>(
+    #[doc(alias = "GST_DEBUG_BIN_TO_DOT_FILE")]
+    #[doc(alias = "gst_debug_bin_to_dot_file")]
+    fn debug_to_dot_file(
         &self,
         details: crate::DebugGraphDetails,
-        file_name: Q,
+        file_name: impl AsRef<path::Path>,
     );
-    fn debug_to_dot_file_with_ts<Q: AsRef<path::Path>>(
+    #[doc(alias = "GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS")]
+    #[doc(alias = "gst_debug_bin_to_dot_file_with_ts")]
+    fn debug_to_dot_file_with_ts(
         &self,
         details: crate::DebugGraphDetails,
-        file_name: Q,
+        file_name: impl AsRef<path::Path>,
     );
 
     fn set_bin_flags(&self, flags: BinFlags);
@@ -166,18 +174,18 @@ impl<O: IsA<Bin>> GstBinExtManual for O {
         crate::debug_bin_to_dot_data(self, details)
     }
 
-    fn debug_to_dot_file<Q: AsRef<path::Path>>(
+    fn debug_to_dot_file(
         &self,
         details: crate::DebugGraphDetails,
-        file_name: Q,
+        file_name: impl AsRef<path::Path>,
     ) {
         crate::debug_bin_to_dot_file(self, details, file_name)
     }
 
-    fn debug_to_dot_file_with_ts<Q: AsRef<path::Path>>(
+    fn debug_to_dot_file_with_ts(
         &self,
         details: crate::DebugGraphDetails,
-        file_name: Q,
+        file_name: impl AsRef<path::Path>,
     ) {
         crate::debug_bin_to_dot_file_with_ts(self, details, file_name)
     }
