@@ -22,13 +22,14 @@ use crate::{SpecificFormattedValue, SpecificFormattedValueIntrinsic};
 use std::cell::RefCell;
 use std::mem;
 use std::num::NonZeroU64;
+use std::ops::ControlFlow;
 use std::ptr;
 
 use glib::ffi::gpointer;
 use glib::prelude::*;
 use glib::translate::*;
 
-#[derive(Debug, PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct PadProbeId(NonZeroU64);
 
 impl IntoGlib for PadProbeId {
@@ -47,6 +48,7 @@ impl FromGlib<libc::c_ulong> for PadProbeId {
     }
 }
 
+#[doc(alias = "GstPadProbeInfo")]
 #[derive(Debug)]
 pub struct PadProbeInfo<'a> {
     pub mask: PadProbeType,
@@ -88,6 +90,13 @@ pub enum PadGetRangeSuccess {
     NewBuffer(crate::Buffer),
 }
 
+#[derive(Debug)]
+pub enum EventForeachAction {
+    Keep,
+    Remove,
+    Replace(Event),
+}
+
 pub trait PadExtManual: 'static {
     #[doc(alias = "gst_pad_add_probe")]
     fn add_probe<F>(&self, mask: PadProbeType, func: F) -> Option<PadProbeId>
@@ -140,11 +149,15 @@ pub trait PadExtManual: 'static {
     fn proxy_query_accept_caps(&self, query: &mut QueryRef) -> bool;
 
     #[doc(alias = "gst_pad_event_default")]
-    fn event_default<P: IsA<crate::Object>>(&self, parent: Option<&P>, event: Event) -> bool;
+    fn event_default<P: IsA<crate::Object>>(
+        &self,
+        parent: Option<&P>,
+        event: impl Into<Event>,
+    ) -> bool;
     #[doc(alias = "gst_pad_push_event")]
-    fn push_event(&self, event: Event) -> bool;
+    fn push_event(&self, event: impl Into<Event>) -> bool;
     #[doc(alias = "gst_pad_send_event")]
-    fn send_event(&self, event: Event) -> bool;
+    fn send_event(&self, event: impl Into<Event>) -> bool;
 
     #[doc(alias = "gst_pad_iterate_internal_links")]
     fn iterate_internal_links(&self) -> crate::Iterator<Pad>;
@@ -156,10 +169,14 @@ pub trait PadExtManual: 'static {
 
     fn stream_lock(&self) -> StreamLock;
 
+    #[doc(alias = "gst_pad_set_activate_function")]
+    #[doc(alias = "gst_pad_set_activate_function_full")]
     unsafe fn set_activate_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>) -> Result<(), LoggableError> + Send + Sync + 'static;
 
+    #[doc(alias = "gst_pad_set_activatemode_function")]
+    #[doc(alias = "gst_pad_set_activatemode_function_full")]
     unsafe fn set_activatemode_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>, crate::PadMode, bool) -> Result<(), LoggableError>
@@ -167,6 +184,8 @@ pub trait PadExtManual: 'static {
             + Sync
             + 'static;
 
+    #[doc(alias = "gst_pad_set_chain_function")]
+    #[doc(alias = "gst_pad_set_chain_function_full")]
     unsafe fn set_chain_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>, crate::Buffer) -> Result<FlowSuccess, FlowError>
@@ -174,6 +193,8 @@ pub trait PadExtManual: 'static {
             + Sync
             + 'static;
 
+    #[doc(alias = "gst_pad_set_chain_list_function")]
+    #[doc(alias = "gst_pad_set_chain_list_function_full")]
     unsafe fn set_chain_list_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>, crate::BufferList) -> Result<FlowSuccess, FlowError>
@@ -181,10 +202,14 @@ pub trait PadExtManual: 'static {
             + Sync
             + 'static;
 
+    #[doc(alias = "gst_pad_set_event_function")]
+    #[doc(alias = "gst_pad_set_event_function_full")]
     unsafe fn set_event_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>, crate::Event) -> bool + Send + Sync + 'static;
 
+    #[doc(alias = "gst_pad_set_event_full_function")]
+    #[doc(alias = "gst_pad_set_event_full_function_full")]
     unsafe fn set_event_full_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>, crate::Event) -> Result<FlowSuccess, FlowError>
@@ -192,6 +217,8 @@ pub trait PadExtManual: 'static {
             + Sync
             + 'static;
 
+    #[doc(alias = "gst_pad_set_getrange_function")]
+    #[doc(alias = "gst_pad_set_getrange_function_full")]
     unsafe fn set_getrange_function<F>(&self, func: F)
     where
         F: Fn(
@@ -205,10 +232,14 @@ pub trait PadExtManual: 'static {
             + Sync
             + 'static;
 
+    #[doc(alias = "gst_pad_set_iterate_internal_links_function")]
+    #[doc(alias = "gst_pad_set_iterate_internal_links_function_full")]
     unsafe fn set_iterate_internal_links_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>) -> crate::Iterator<Pad> + Send + Sync + 'static;
 
+    #[doc(alias = "gst_pad_set_link_function")]
+    #[doc(alias = "gst_pad_set_link_function_full")]
     unsafe fn set_link_function<F>(&self, func: F)
     where
         F: Fn(
@@ -220,10 +251,14 @@ pub trait PadExtManual: 'static {
             + Sync
             + 'static;
 
+    #[doc(alias = "gst_pad_set_query_function")]
+    #[doc(alias = "gst_pad_set_query_function_full")]
     unsafe fn set_query_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>, &mut crate::QueryRef) -> bool + Send + Sync + 'static;
 
+    #[doc(alias = "gst_pad_set_unlink_function")]
+    #[doc(alias = "gst_pad_set_unlink_function_full")]
     unsafe fn set_unlink_function<F>(&self, func: F)
     where
         F: Fn(&Self, Option<&crate::Object>) + Send + Sync + 'static;
@@ -236,6 +271,7 @@ pub trait PadExtManual: 'static {
         &self,
         src_val: V,
     ) -> Option<U>;
+    #[doc(alias = "gst_pad_peer_query_convert")]
     fn peer_query_convert_generic<V: Into<GenericFormattedValue>>(
         &self,
         src_val: V,
@@ -244,10 +280,12 @@ pub trait PadExtManual: 'static {
 
     #[doc(alias = "gst_pad_peer_query_duration")]
     fn peer_query_duration<T: SpecificFormattedValueIntrinsic>(&self) -> Option<T>;
+    #[doc(alias = "gst_pad_peer_query_duration")]
     fn peer_query_duration_generic(&self, format: Format) -> Option<GenericFormattedValue>;
 
     #[doc(alias = "gst_pad_peer_query_position")]
     fn peer_query_position<T: SpecificFormattedValueIntrinsic>(&self) -> Option<T>;
+    #[doc(alias = "gst_pad_peer_query_position")]
     fn peer_query_position_generic(&self, format: Format) -> Option<GenericFormattedValue>;
 
     #[doc(alias = "gst_pad_query_convert")]
@@ -255,6 +293,7 @@ pub trait PadExtManual: 'static {
         &self,
         src_val: V,
     ) -> Option<U>;
+    #[doc(alias = "gst_pad_query_convert")]
     fn query_convert_generic<V: Into<GenericFormattedValue>>(
         &self,
         src_val: V,
@@ -263,21 +302,30 @@ pub trait PadExtManual: 'static {
 
     #[doc(alias = "gst_pad_query_duration")]
     fn query_duration<T: SpecificFormattedValueIntrinsic>(&self) -> Option<T>;
+    #[doc(alias = "gst_pad_query_duration")]
     fn query_duration_generic(&self, format: Format) -> Option<GenericFormattedValue>;
 
     #[doc(alias = "gst_pad_query_position")]
     fn query_position<T: SpecificFormattedValueIntrinsic>(&self) -> Option<T>;
+    #[doc(alias = "gst_pad_query_position")]
     fn query_position_generic(&self, format: Format) -> Option<GenericFormattedValue>;
 
     #[doc(alias = "get_mode")]
+    #[doc(alias = "GST_PAD_MODE")]
     fn mode(&self) -> crate::PadMode;
 
     #[doc(alias = "gst_pad_sticky_events_foreach")]
-    fn sticky_events_foreach<F: FnMut(Event) -> Result<Option<Event>, Option<Event>>>(
+    fn sticky_events_foreach<
+        F: FnMut(&Event) -> ControlFlow<EventForeachAction, EventForeachAction>,
+    >(
         &self,
         func: F,
     );
 
+    #[doc(alias = "gst_pad_get_sticky_event")]
+    #[doc(alias = "get_sticky_event")]
+    fn sticky_event<T: crate::event::StickyEventType>(&self, idx: u32) -> Option<T>;
+
     fn set_pad_flags(&self, flags: PadFlags);
 
     fn unset_pad_flags(&self, flags: PadFlags);
@@ -482,31 +530,35 @@ impl<O: IsA<Pad>> PadExtManual for O {
         }
     }
 
-    fn event_default<P: IsA<crate::Object>>(&self, parent: Option<&P>, event: Event) -> bool {
+    fn event_default<P: IsA<crate::Object>>(
+        &self,
+        parent: Option<&P>,
+        event: impl Into<Event>,
+    ) -> bool {
         skip_assert_initialized!();
         unsafe {
             from_glib(ffi::gst_pad_event_default(
                 self.as_ref().to_glib_none().0,
                 parent.map(|p| p.as_ref()).to_glib_none().0,
-                event.into_ptr(),
+                event.into().into_ptr(),
             ))
         }
     }
 
-    fn push_event(&self, event: Event) -> bool {
+    fn push_event(&self, event: impl Into<Event>) -> bool {
         unsafe {
             from_glib(ffi::gst_pad_push_event(
                 self.as_ref().to_glib_none().0,
-                event.into_ptr(),
+                event.into().into_ptr(),
             ))
         }
     }
 
-    fn send_event(&self, event: Event) -> bool {
+    fn send_event(&self, event: impl Into<Event>) -> bool {
         unsafe {
             from_glib(ffi::gst_pad_send_event(
                 self.as_ref().to_glib_none().0,
-                event.into_ptr(),
+                event.into().into_ptr(),
             ))
         }
     }
@@ -957,55 +1009,72 @@ impl<O: IsA<Pad>> PadExtManual for O {
         }
     }
 
-    fn sticky_events_foreach<F: FnMut(Event) -> Result<Option<Event>, Option<Event>>>(
+    fn sticky_events_foreach<
+        F: FnMut(&Event) -> ControlFlow<EventForeachAction, EventForeachAction>,
+    >(
         &self,
         func: F,
     ) {
-        unsafe extern "C" fn trampoline(
+        unsafe extern "C" fn trampoline<
+            F: FnMut(&Event) -> ControlFlow<EventForeachAction, EventForeachAction>,
+        >(
             _pad: *mut ffi::GstPad,
             event: *mut *mut ffi::GstEvent,
             user_data: glib::ffi::gpointer,
         ) -> glib::ffi::gboolean {
-            let func =
-                user_data as *mut &mut (dyn FnMut(Event) -> Result<Option<Event>, Option<Event>>);
-            let res = (*func)(from_glib_full(*event));
+            let func = user_data as *mut F;
+            let res = (*func)(&from_glib_borrow(*event));
 
-            match res {
-                Ok(Some(ev)) => {
-                    *event = ev.into_ptr();
-                    glib::ffi::GTRUE
-                }
-                Err(Some(ev)) => {
-                    *event = ev.into_ptr();
-                    glib::ffi::GFALSE
-                }
-                Ok(None) => {
+            let (do_continue, ev_action) = match res {
+                ControlFlow::Continue(ev_action) => (glib::ffi::GTRUE, ev_action),
+                ControlFlow::Break(ev_action) => (glib::ffi::GFALSE, ev_action),
+            };
+
+            use EventForeachAction::*;
+
+            match ev_action {
+                Keep => (), // do nothing
+                Remove => {
+                    ffi::gst_mini_object_unref(*event as *mut _);
                     *event = ptr::null_mut();
-                    glib::ffi::GTRUE
                 }
-                Err(None) => {
-                    *event = ptr::null_mut();
-                    glib::ffi::GFALSE
+                Replace(ev) => {
+                    ffi::gst_mini_object_unref(*event as *mut _);
+                    *event = ev.into_ptr();
                 }
             }
+
+            do_continue
         }
 
         unsafe {
             let mut func = func;
-            let func_obj: &mut (dyn FnMut(Event) -> Result<Option<Event>, Option<Event>>) =
-                &mut func;
-            let func_ptr = &func_obj
-                as *const &mut (dyn FnMut(Event) -> Result<Option<Event>, Option<Event>>)
-                as glib::ffi::gpointer;
+            let func_ptr = &mut func as *mut F as glib::ffi::gpointer;
 
             ffi::gst_pad_sticky_events_foreach(
                 self.as_ref().to_glib_none().0,
-                Some(trampoline),
+                Some(trampoline::<F>),
                 func_ptr,
             );
         }
     }
 
+    fn sticky_event<T: crate::event::StickyEventType>(&self, idx: u32) -> Option<T> {
+        unsafe {
+            let ptr = ffi::gst_pad_get_sticky_event(
+                self.as_ref().to_glib_none().0,
+                T::TYPE.into_glib(),
+                idx,
+            );
+
+            if ptr.is_null() {
+                None
+            } else {
+                Some(T::from_event(from_glib_full(ptr)))
+            }
+        }
+    }
+
     fn set_pad_flags(&self, flags: PadFlags) {
         unsafe {
             let ptr: *mut ffi::GstObject = self.as_ptr() as *mut _;
@@ -1331,6 +1400,7 @@ where
     res.into_glib()
 }
 
+#[allow(clippy::needless_option_as_deref)]
 unsafe extern "C" fn trampoline_getrange_function<
     T,
     F: Fn(
@@ -1542,21 +1612,25 @@ unsafe extern "C" fn destroy_closure_pad_task<F>(ptr: gpointer) {
 }
 
 impl Pad {
+    #[doc(alias = "gst_pad_new")]
     pub fn new(name: Option<&str>, direction: crate::PadDirection) -> Self {
         skip_assert_initialized!();
         Self::builder(name, direction).build()
     }
 
+    #[doc(alias = "gst_pad_new")]
     pub fn builder(name: Option<&str>, direction: crate::PadDirection) -> PadBuilder<Self> {
         skip_assert_initialized!();
         PadBuilder::new(name, direction)
     }
 
+    #[doc(alias = "gst_pad_new_from_static_template")]
     pub fn from_static_template(templ: &StaticPadTemplate, name: Option<&str>) -> Self {
         skip_assert_initialized!();
         Self::builder_with_static_template(templ, name).build()
     }
 
+    #[doc(alias = "gst_pad_new_from_static_template")]
     pub fn builder_with_static_template(
         templ: &StaticPadTemplate,
         name: Option<&str>,
@@ -1565,11 +1639,13 @@ impl Pad {
         PadBuilder::from_static_template(templ, name)
     }
 
+    #[doc(alias = "gst_pad_new_from_template")]
     pub fn from_template(templ: &crate::PadTemplate, name: Option<&str>) -> Self {
         skip_assert_initialized!();
         Self::builder_with_template(templ, name).build()
     }
 
+    #[doc(alias = "gst_pad_new_from_template")]
     pub fn builder_with_template(
         templ: &crate::PadTemplate,
         name: Option<&str>,
@@ -1579,6 +1655,7 @@ impl Pad {
     }
 }
 
+#[must_use = "The builder must be built to be used"]
 pub struct PadBuilder<T>(pub(crate) T);
 
 impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
@@ -1615,11 +1692,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         // Since 1.14 templates can keep a pad GType with them, so we need to do some
         // additional checks here now
         if templ.has_property("gtype", Some(glib::Type::static_type())) {
-            let gtype = templ
-                .property("gtype")
-                .unwrap()
-                .get::<glib::Type>()
-                .unwrap();
+            let gtype = templ.property::<glib::Type>("gtype");
 
             if gtype == glib::Type::UNIT {
                 // Nothing to be done, we can create any kind of pad
@@ -1657,6 +1730,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         PadBuilder(pad)
     }
 
+    #[doc(alias = "gst_pad_set_activate_function")]
     pub fn activate_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>) -> Result<(), LoggableError> + Send + Sync + 'static,
@@ -1668,6 +1742,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_activatemode_function")]
     pub fn activatemode_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>, crate::PadMode, bool) -> Result<(), LoggableError>
@@ -1682,6 +1757,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_chain_function")]
     pub fn chain_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>, crate::Buffer) -> Result<FlowSuccess, FlowError>
@@ -1696,6 +1772,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_chain_list_function")]
     pub fn chain_list_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>, crate::BufferList) -> Result<FlowSuccess, FlowError>
@@ -1710,6 +1787,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_event_function")]
     pub fn event_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>, crate::Event) -> bool + Send + Sync + 'static,
@@ -1721,6 +1799,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_event_full_function")]
     pub fn event_full_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>, crate::Event) -> Result<FlowSuccess, FlowError>
@@ -1735,6 +1814,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_getrange_function")]
     pub fn getrange_function<F>(self, func: F) -> Self
     where
         F: Fn(
@@ -1755,6 +1835,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_iterate_internal_links_function")]
     pub fn iterate_internal_links_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>) -> crate::Iterator<Pad> + Send + Sync + 'static,
@@ -1766,6 +1847,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_link_function")]
     pub fn link_function<F>(self, func: F) -> Self
     where
         F: Fn(
@@ -1784,6 +1866,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_query_function")]
     pub fn query_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>, &mut crate::QueryRef) -> bool + Send + Sync + 'static,
@@ -1795,6 +1878,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[doc(alias = "gst_pad_set_unlink_function")]
     pub fn unlink_function<F>(self, func: F) -> Self
     where
         F: Fn(&T, Option<&crate::Object>) + Send + Sync + 'static,
@@ -1812,6 +1896,7 @@ impl<T: IsA<Pad> + IsA<glib::Object> + glib::object::IsClass> PadBuilder<T> {
         self
     }
 
+    #[must_use = "Building the pad without using it has no effect"]
     pub fn build(self) -> T {
         self.0
     }
@@ -2083,4 +2168,108 @@ mod tests {
             "An event ref leaked!"
         );
     }
+
+    #[test]
+    fn test_sticky_events_foreach() {
+        crate::init().unwrap();
+
+        let pad = crate::Pad::builder(Some("sink"), crate::PadDirection::Sink).build();
+        pad.set_active(true).unwrap();
+
+        // Send some sticky events
+        assert!(pad.send_event(crate::event::StreamStart::new("test")));
+
+        let caps = crate::Caps::builder("some/x-caps").build();
+        assert!(pad.send_event(crate::event::Caps::new(&caps)));
+
+        let segment = crate::FormattedSegment::<crate::ClockTime>::new();
+        assert!(pad.send_event(crate::event::Segment::new(segment.as_ref())));
+
+        let mut sticky_events = Vec::new();
+        pad.sticky_events_foreach(|event| {
+            sticky_events.push(event.clone());
+            ControlFlow::Continue(EventForeachAction::Keep)
+        });
+        assert_eq!(sticky_events.len(), 3);
+
+        // Test early exit from foreach loop
+        let mut sticky_events2 = Vec::new();
+        pad.sticky_events_foreach(|event| {
+            sticky_events2.push(event.clone());
+            if event.type_() == crate::EventType::Caps {
+                ControlFlow::Break(EventForeachAction::Keep)
+            } else {
+                ControlFlow::Continue(EventForeachAction::Keep)
+            }
+        });
+        assert_eq!(sticky_events2.len(), 2);
+
+        let mut sticky_events3 = Vec::new();
+        pad.sticky_events_foreach(|event| {
+            sticky_events3.push(event.clone());
+            ControlFlow::Continue(EventForeachAction::Keep)
+        });
+        assert_eq!(sticky_events3.len(), 3);
+
+        for (e1, e2) in sticky_events.iter().zip(sticky_events3.iter()) {
+            assert_eq!(e1.as_ref() as *const _, e2.as_ref() as *const _);
+        }
+
+        // Replace segment event
+        pad.sticky_events_foreach(|event| {
+            let action = if event.type_() == crate::EventType::Segment {
+                let byte_segment = crate::FormattedSegment::<crate::format::Bytes>::new();
+                EventForeachAction::Replace(crate::event::Segment::new(&byte_segment))
+            } else {
+                EventForeachAction::Keep
+            };
+            ControlFlow::Continue(action)
+        });
+
+        // Check that segment event is different now
+        let mut sticky_events4 = Vec::new();
+        pad.sticky_events_foreach(|event| {
+            sticky_events4.push(event.clone());
+            ControlFlow::Continue(EventForeachAction::Keep)
+        });
+        assert_eq!(sticky_events4.len(), 3);
+        assert_eq!(
+            sticky_events[0].as_ref() as *const _,
+            sticky_events4[0].as_ref() as *const _
+        );
+        assert_eq!(
+            sticky_events[1].as_ref() as *const _,
+            sticky_events4[1].as_ref() as *const _
+        );
+        assert_ne!(
+            sticky_events[2].as_ref() as *const _,
+            sticky_events4[2].as_ref() as *const _
+        );
+
+        // Drop caps event
+        pad.sticky_events_foreach(|event| {
+            let action = if event.type_() == crate::EventType::Caps {
+                EventForeachAction::Remove
+            } else {
+                EventForeachAction::Keep
+            };
+            ControlFlow::Continue(action)
+        });
+
+        // Check that caps event actually got removed
+        let mut sticky_events5 = Vec::new();
+        pad.sticky_events_foreach(|event| {
+            sticky_events5.push(event.clone());
+            ControlFlow::Continue(EventForeachAction::Keep)
+        });
+        assert_eq!(sticky_events5.len(), 2);
+        assert_eq!(
+            sticky_events4[0].as_ref() as *const _,
+            sticky_events5[0].as_ref() as *const _
+        );
+        assert_eq!(
+            sticky_events4[2].as_ref() as *const _,
+            sticky_events5[1].as_ref() as *const _
+        );
+    }
 }
