@@ -63,11 +63,13 @@ impl GlibValueError for glib::value::ValueTypeMismatchOrNoneError {
 }
 
 #[doc(alias = "GstStructure")]
+#[repr(transparent)]
 pub struct Structure(ptr::NonNull<ffi::GstStructure>);
 unsafe impl Send for Structure {}
 unsafe impl Sync for Structure {}
 
 impl Structure {
+    #[doc(alias = "gst_structure_new")]
     pub fn builder(name: &str) -> Builder {
         assert_initialized_main_thread!();
         Builder::new(name)
@@ -83,6 +85,7 @@ impl Structure {
         }
     }
 
+    #[doc(alias = "gst_structure_new")]
     pub fn new(name: &str, values: &[(&str, &(dyn ToSendValue + Sync))]) -> Structure {
         assert_initialized_main_thread!();
         let mut structure = Structure::new_empty(name);
@@ -100,17 +103,15 @@ impl Structure {
     }
 
     #[allow(clippy::should_implement_trait)]
-    pub fn from_iter<'a, 'b, I>(name: &str, iter: I) -> Structure
-    where
-        I: IntoIterator<Item = (&'a str, &'b SendValue)>,
-    {
+    pub fn from_iter<'a>(
+        name: &str,
+        iter: impl IntoIterator<Item = (&'a str, SendValue)>,
+    ) -> Structure {
         assert_initialized_main_thread!();
         let mut structure = Structure::new_empty(name);
 
-        iter.into_iter().for_each(|(f, v)| unsafe {
-            let mut value = v.clone().into_raw();
-            ffi::gst_structure_take_value(structure.0.as_mut(), f.to_glib_none().0, &mut value);
-        });
+        iter.into_iter()
+            .for_each(|(f, v)| structure.set_value(f, v));
 
         structure
     }
@@ -306,6 +307,8 @@ impl glib::value::ValueType for Structure {
     type Type = Self;
 }
 
+impl glib::value::ValueTypeOptional for Structure {}
+
 unsafe impl<'a> glib::value::FromValue<'a> for Structure {
     type Checker = glib::value::GenericValueTypeOrNoneChecker<Self>;
 
@@ -382,37 +385,42 @@ impl StructureRef {
         self as *const Self as *mut ffi::GstStructure
     }
 
+    #[doc(alias = "gst_structure_get")]
     pub fn get<'a, T: FromValue<'a>>(&'a self, name: &str) -> Result<T, GetError>
     where
         <<T as FromValue<'a>>::Checker as glib::value::ValueTypeChecker>::Error: GlibValueError,
     {
-        let name = glib::Quark::from_string(name);
+        let name = glib::Quark::from_str(name);
         self.get_by_quark(name)
     }
 
+    #[doc(alias = "gst_structure_get")]
     pub fn get_optional<'a, T: FromValue<'a>>(&'a self, name: &str) -> Result<Option<T>, GetError>
     where
         <<T as FromValue<'a>>::Checker as glib::value::ValueTypeChecker>::Error: GlibValueError,
     {
-        let name = glib::Quark::from_string(name);
+        let name = glib::Quark::from_str(name);
         self.get_optional_by_quark(name)
     }
 
     #[doc(alias = "get_value")]
+    #[doc(alias = "gst_structure_get_value")]
     pub fn value(&self, name: &str) -> Result<&SendValue, GetError> {
-        let name = glib::Quark::from_string(name);
+        let name = glib::Quark::from_str(name);
         self.value_by_quark(name)
     }
 
+    #[doc(alias = "gst_structure_id_get")]
     pub fn get_by_quark<'a, T: FromValue<'a>>(&'a self, name: glib::Quark) -> Result<T, GetError>
     where
         <<T as FromValue<'a>>::Checker as glib::value::ValueTypeChecker>::Error: GlibValueError,
     {
         self.value_by_quark(name)?
             .get()
-            .map_err(|err| GetError::from_value_get_error(name.to_string(), err))
+            .map_err(|err| GetError::from_value_get_error(name.as_str(), err))
     }
 
+    #[doc(alias = "gst_structure_id_get")]
     pub fn get_optional_by_quark<'a, T: FromValue<'a>>(
         &'a self,
         name: glib::Quark,
@@ -424,42 +432,49 @@ impl StructureRef {
             .ok()
             .map(|v| v.get())
             .transpose()
-            .map_err(|err| GetError::from_value_get_error(name.to_string(), err))
+            .map_err(|err| GetError::from_value_get_error(name.as_str(), err))
     }
 
+    #[doc(alias = "gst_structure_id_get_value")]
     pub fn value_by_quark(&self, name: glib::Quark) -> Result<&SendValue, GetError> {
         unsafe {
             let value = ffi::gst_structure_id_get_value(&self.0, name.into_glib());
 
             if value.is_null() {
-                return Err(GetError::new_field_not_found(name.to_string()));
+                return Err(GetError::new_field_not_found(name.as_str()));
             }
 
             Ok(&*(value as *const SendValue))
         }
     }
 
+    #[doc(alias = "gst_structure_set")]
     pub fn set<T: ToSendValue + Sync>(&mut self, name: &str, value: T) {
         let value = value.to_send_value();
         self.set_value(name, value);
     }
 
+    #[doc(alias = "gst_structure_set_value")]
     pub fn set_value(&mut self, name: &str, value: SendValue) {
         unsafe {
-            let mut value = value.into_raw();
-            ffi::gst_structure_take_value(&mut self.0, name.to_glib_none().0, &mut value);
+            ffi::gst_structure_take_value(
+                &mut self.0,
+                name.to_glib_none().0,
+                &mut value.into_raw(),
+            );
         }
     }
 
+    #[doc(alias = "gst_structure_id_set")]
     pub fn set_by_quark<T: ToSendValue + Sync>(&mut self, name: glib::Quark, value: T) {
         let value = value.to_send_value();
         self.set_value_by_quark(name, value);
     }
 
+    #[doc(alias = "gst_structure_id_set_value")]
     pub fn set_value_by_quark(&mut self, name: glib::Quark, value: SendValue) {
         unsafe {
-            let mut value = value.into_raw();
-            ffi::gst_structure_id_take_value(&mut self.0, name.into_glib(), &mut value);
+            ffi::gst_structure_id_take_value(&mut self.0, name.into_glib(), &mut value.into_raw());
         }
     }
 
@@ -473,6 +488,7 @@ impl StructureRef {
         }
     }
 
+    #[doc(alias = "gst_structure_get_name_id")]
     pub fn name_quark(&self) -> glib::Quark {
         unsafe { from_glib(ffi::gst_structure_get_name_id(&self.0)) }
     }
@@ -482,6 +498,11 @@ impl StructureRef {
         unsafe { ffi::gst_structure_set_name(&mut self.0, name.to_glib_none().0) }
     }
 
+    #[doc(alias = "gst_structure_has_name")]
+    pub fn has_name(&self, name: &str) -> bool {
+        self.name() == name
+    }
+
     #[doc(alias = "gst_structure_has_field")]
     pub fn has_field(&self, field: &str) -> bool {
         unsafe {
@@ -492,6 +513,7 @@ impl StructureRef {
         }
     }
 
+    #[doc(alias = "gst_structure_has_field_typed")]
     pub fn has_field_with_type(&self, field: &str, type_: glib::Type) -> bool {
         unsafe {
             from_glib(ffi::gst_structure_has_field_typed(
@@ -502,10 +524,12 @@ impl StructureRef {
         }
     }
 
+    #[doc(alias = "gst_structure_id_has_field")]
     pub fn has_field_by_quark(&self, field: glib::Quark) -> bool {
         unsafe { from_glib(ffi::gst_structure_id_has_field(&self.0, field.into_glib())) }
     }
 
+    #[doc(alias = "gst_structure_id_has_field_typed")]
     pub fn has_field_with_type_by_quark(&self, field: glib::Quark, type_: glib::Type) -> bool {
         unsafe {
             from_glib(ffi::gst_structure_id_has_field_typed(
@@ -523,6 +547,7 @@ impl StructureRef {
         }
     }
 
+    #[doc(alias = "gst_structure_remove_fields")]
     pub fn remove_fields(&mut self, fields: &[&str]) {
         for f in fields {
             self.remove_field(f)
@@ -638,8 +663,8 @@ impl StructureRef {
             from_glib(ffi::gst_structure_fixate_field_nearest_fraction(
                 &mut self.0,
                 name.to_glib_none().0,
-                *target.numer(),
-                *target.denom(),
+                target.numer(),
+                target.denom(),
             ))
         }
     }
@@ -661,6 +686,97 @@ impl StructureRef {
     pub fn serialize(&self, flags: crate::SerializeFlags) -> glib::GString {
         unsafe { from_glib_full(ffi::gst_structure_serialize(&self.0, flags.into_glib())) }
     }
+
+    #[doc(alias = "gst_structure_foreach")]
+    pub fn foreach<F: FnMut(glib::Quark, &glib::Value) -> std::ops::ControlFlow<()>>(
+        &self,
+        mut func: F,
+    ) -> bool {
+        unsafe {
+            unsafe extern "C" fn trampoline<
+                F: FnMut(glib::Quark, &glib::Value) -> std::ops::ControlFlow<()>,
+            >(
+                quark: glib::ffi::GQuark,
+                value: *const glib::gobject_ffi::GValue,
+                user_data: glib::ffi::gpointer,
+            ) -> glib::ffi::gboolean {
+                let func = &mut *(user_data as *mut F);
+                let res = func(from_glib(quark), &*(value as *const glib::Value));
+
+                matches!(res, std::ops::ControlFlow::Continue(_)).into_glib()
+            }
+            let func = &mut func as *mut F;
+            from_glib(ffi::gst_structure_foreach(
+                self.as_ptr(),
+                Some(trampoline::<F>),
+                func as glib::ffi::gpointer,
+            ))
+        }
+    }
+
+    #[doc(alias = "gst_structure_map_in_place")]
+    pub fn map_in_place<F: FnMut(glib::Quark, &mut glib::Value) -> std::ops::ControlFlow<()>>(
+        &mut self,
+        mut func: F,
+    ) -> bool {
+        unsafe {
+            unsafe extern "C" fn trampoline<
+                F: FnMut(glib::Quark, &mut glib::Value) -> std::ops::ControlFlow<()>,
+            >(
+                quark: glib::ffi::GQuark,
+                value: *mut glib::gobject_ffi::GValue,
+                user_data: glib::ffi::gpointer,
+            ) -> glib::ffi::gboolean {
+                let func = &mut *(user_data as *mut F);
+                let res = func(from_glib(quark), &mut *(value as *mut glib::Value));
+
+                matches!(res, std::ops::ControlFlow::Continue(_)).into_glib()
+            }
+            let func = &mut func as *mut F;
+            from_glib(ffi::gst_structure_map_in_place(
+                self.as_mut_ptr(),
+                Some(trampoline::<F>),
+                func as glib::ffi::gpointer,
+            ))
+        }
+    }
+
+    #[doc(alias = "gst_structure_filter_and_map_in_place")]
+    pub fn filter_map_in_place<F: FnMut(glib::Quark, glib::Value) -> Option<glib::Value>>(
+        &mut self,
+        mut func: F,
+    ) {
+        unsafe {
+            unsafe extern "C" fn trampoline<
+                F: FnMut(glib::Quark, glib::Value) -> Option<glib::Value>,
+            >(
+                quark: glib::ffi::GQuark,
+                value: *mut glib::gobject_ffi::GValue,
+                user_data: glib::ffi::gpointer,
+            ) -> glib::ffi::gboolean {
+                let func = &mut *(user_data as *mut F);
+
+                let v = mem::replace(
+                    &mut *(value as *mut glib::Value),
+                    glib::Value::uninitialized(),
+                );
+                match func(from_glib(quark), v) {
+                    None => glib::ffi::GFALSE,
+                    Some(v) => {
+                        *value = v.into_raw();
+                        glib::ffi::GTRUE
+                    }
+                }
+            }
+
+            let func = &mut func as *mut F;
+            ffi::gst_structure_filter_and_map_in_place(
+                self.as_mut_ptr(),
+                Some(trampoline::<F>),
+                func as glib::ffi::gpointer,
+            );
+        }
+    }
 }
 
 impl fmt::Display for StructureRef {
@@ -835,7 +951,36 @@ impl<'a> DoubleEndedIterator for Iter<'a> {
 
 impl<'a> ExactSizeIterator for Iter<'a> {}
 
+impl<'a> IntoIterator for &'a StructureRef {
+    type IntoIter = Iter<'a>;
+    type Item = (&'static str, &'a SendValue);
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.iter()
+    }
+}
+
+impl<'a> std::iter::Extend<(&'a str, SendValue)> for StructureRef {
+    fn extend<T: IntoIterator<Item = (&'a str, SendValue)>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|(f, v)| self.set_value(f, v));
+    }
+}
+
+impl std::iter::Extend<(String, SendValue)> for StructureRef {
+    fn extend<T: IntoIterator<Item = (String, SendValue)>>(&mut self, iter: T) {
+        iter.into_iter().for_each(|(f, v)| self.set_value(&f, v));
+    }
+}
+
+impl std::iter::Extend<(glib::Quark, SendValue)> for StructureRef {
+    fn extend<T: IntoIterator<Item = (glib::Quark, SendValue)>>(&mut self, iter: T) {
+        iter.into_iter()
+            .for_each(|(f, v)| self.set_value_by_quark(f, v));
+    }
+}
+
 #[derive(Debug)]
+#[must_use = "The builder must be built to be used"]
 pub struct Builder {
     s: Structure,
 }
@@ -853,6 +998,7 @@ impl Builder {
         self
     }
 
+    #[must_use = "Building the structure without using it has no effect"]
     pub fn build(self) -> Structure {
         self.s
     }
@@ -861,7 +1007,6 @@ impl Builder {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use std::str::FromStr;
 
     #[test]
     fn new_set_get() {
@@ -872,9 +1017,9 @@ mod tests {
         let mut s = Structure::new_empty("test");
         assert_eq!(s.name(), "test");
 
-        s.set("f1", &"abc");
+        s.set("f1", "abc");
         s.set("f2", &String::from("bcd"));
-        s.set("f3", &123i32);
+        s.set("f3", 123i32);
 
         assert_eq!(s.get::<&str>("f1"), Ok("abc"));
         assert_eq!(s.get::<Option<&str>>("f2"), Ok(Some("bcd")));
@@ -924,9 +1069,9 @@ mod tests {
         crate::init().unwrap();
 
         let s = Structure::builder("test")
-            .field("f1", &"abc")
+            .field("f1", "abc")
             .field("f2", &String::from("bcd"))
-            .field("f3", &123i32)
+            .field("f3", 123i32)
             .build();
 
         assert_eq!(s.name(), "test");
@@ -941,7 +1086,7 @@ mod tests {
 
         let a = "Test, f1=(string)abc, f2=(uint)123;";
 
-        let s = Structure::from_str(a).unwrap();
+        let s = a.parse::<Structure>().unwrap();
         assert_eq!(s.get::<&str>("f1"), Ok("abc"));
         assert_eq!(s.get::<u32>("f2"), Ok(123));
 
@@ -956,7 +1101,7 @@ mod tests {
 
         let a = None::<&Structure>.to_value();
         assert!(a.get::<Option<Structure>>().unwrap().is_none());
-        let b = Structure::from_str("foo").unwrap().to_value();
+        let b = "foo".parse::<Structure>().unwrap().to_value();
         assert!(b.get::<Option<Structure>>().unwrap().is_some());
     }
 
@@ -965,12 +1110,17 @@ mod tests {
         crate::init().unwrap();
 
         let s = Structure::builder("test")
-            .field("f1", &"abc")
+            .field("f1", "abc")
             .field("f2", &String::from("bcd"))
-            .field("f3", &123i32)
+            .field("f3", 123i32)
             .build();
 
-        let s2 = Structure::from_iter(s.name(), s.iter().filter(|(f, _)| *f == "f1"));
+        let s2 = Structure::from_iter(
+            s.name(),
+            s.iter()
+                .filter(|(f, _)| *f == "f1")
+                .map(|(f, v)| (f, v.clone())),
+        );
 
         assert_eq!(s2.name(), "test");
         assert_eq!(s2.get::<&str>("f1"), Ok("abc"));
