@@ -1,5 +1,6 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+use super::prelude::*;
 use crate::prelude::*;
 use glib::subclass::prelude::*;
 use glib::translate::*;
@@ -13,25 +14,26 @@ use crate::StateChangeError;
 use crate::StateChangeReturn;
 use crate::StateChangeSuccess;
 
+use std::borrow::Cow;
 use std::sync::atomic;
 
 #[derive(Debug, Clone)]
 pub struct ElementMetadata {
-    long_name: String,
-    classification: String,
-    description: String,
-    author: String,
-    additional: Vec<(String, String)>,
+    long_name: Cow<'static, str>,
+    classification: Cow<'static, str>,
+    description: Cow<'static, str>,
+    author: Cow<'static, str>,
+    additional: Cow<'static, [(Cow<'static, str>, Cow<'static, str>)]>,
 }
 
 impl ElementMetadata {
     pub fn new(long_name: &str, classification: &str, description: &str, author: &str) -> Self {
         Self {
-            long_name: long_name.into(),
-            classification: classification.into(),
-            description: description.into(),
-            author: author.into(),
-            additional: vec![],
+            long_name: Cow::Owned(long_name.into()),
+            classification: Cow::Owned(classification.into()),
+            description: Cow::Owned(description.into()),
+            author: Cow::Owned(author.into()),
+            additional: Cow::Borrowed(&[]),
         }
     }
 
@@ -43,20 +45,36 @@ impl ElementMetadata {
         additional: &[(&str, &str)],
     ) -> Self {
         Self {
-            long_name: long_name.into(),
-            classification: classification.into(),
-            description: description.into(),
-            author: author.into(),
+            long_name: Cow::Owned(long_name.into()),
+            classification: Cow::Owned(classification.into()),
+            description: Cow::Owned(description.into()),
+            author: Cow::Owned(author.into()),
             additional: additional
                 .iter()
                 .copied()
-                .map(|(key, value)| (key.into(), value.into()))
+                .map(|(key, value)| (Cow::Owned(key.into()), Cow::Owned(value.into())))
                 .collect(),
         }
     }
+
+    pub const fn with_cow(
+        long_name: Cow<'static, str>,
+        classification: Cow<'static, str>,
+        description: Cow<'static, str>,
+        author: Cow<'static, str>,
+        additional: Cow<'static, [(Cow<'static, str>, Cow<'static, str>)]>,
+    ) -> Self {
+        Self {
+            long_name,
+            classification,
+            description,
+            author,
+            additional,
+        }
+    }
 }
 
-pub trait ElementImpl: ElementImplExt + ObjectImpl + Send + Sync {
+pub trait ElementImpl: ElementImplExt + GstObjectImpl + Send + Sync {
     fn metadata() -> Option<&'static ElementMetadata> {
         None
     }
@@ -330,7 +348,7 @@ impl<T: ElementImpl> ElementImplExt for T {
             assert!(element.type_().is_a(T::type_()));
             let ptr: *mut ffi::GstElement = element.as_ptr() as *mut _;
             let instance = &*(ptr as *mut T::Instance);
-            let imp = instance.impl_();
+            let imp = instance.imp();
 
             panic_to_error!(element, imp.panicked(), fallback(), { f(imp) })
         }
@@ -346,7 +364,7 @@ impl<T: ElementImpl> ElementImplExt for T {
             assert!(wrap.type_().is_a(Self::type_()));
             let ptr: *mut ffi::GstElement = wrap.to_glib_none().0;
             let instance = &*(ptr as *mut Self::Instance);
-            let imp = instance.impl_();
+            let imp = instance.imp();
 
             panic_to_error!(wrap, imp.panicked(), fallback(), {
                 f(imp, wrap.unsafe_cast_ref())
@@ -357,8 +375,7 @@ impl<T: ElementImpl> ElementImplExt for T {
 
 unsafe impl<T: ElementImpl> IsSubclassable<T> for Element {
     fn class_init(klass: &mut glib::Class<Self>) {
-        <glib::Object as IsSubclassable<T>>::class_init(klass);
-
+        Self::parent_class_init::<T>(klass);
         let klass = klass.as_mut();
         klass.change_state = Some(element_change_state::<T>);
         klass.request_new_pad = Some(element_request_new_pad::<T>);
@@ -384,7 +401,7 @@ unsafe impl<T: ElementImpl> IsSubclassable<T> for Element {
                     metadata.author.to_glib_none().0,
                 );
 
-                for (key, value) in &metadata.additional {
+                for (key, value) in &metadata.additional[..] {
                     ffi::gst_element_class_add_metadata(
                         klass,
                         key.to_glib_none().0,
@@ -396,7 +413,7 @@ unsafe impl<T: ElementImpl> IsSubclassable<T> for Element {
     }
 
     fn instance_init(instance: &mut glib::subclass::InitializingObject<T>) {
-        <glib::Object as IsSubclassable<T>>::instance_init(instance);
+        Self::parent_instance_init::<T>(instance);
 
         instance.set_instance_data(Self::static_type(), atomic::AtomicBool::new(false));
     }
@@ -407,7 +424,7 @@ unsafe extern "C" fn element_change_state<T: ElementImpl>(
     transition: ffi::GstStateChange,
 ) -> ffi::GstStateChangeReturn {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     // *Never* fail downwards state changes, this causes bugs in GStreamer
@@ -433,7 +450,7 @@ unsafe extern "C" fn element_request_new_pad<T: ElementImpl>(
     caps: *const ffi::GstCaps,
 ) -> *mut ffi::GstPad {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     let caps = Option::<crate::Caps>::from_glib_borrow(caps);
@@ -467,7 +484,7 @@ unsafe extern "C" fn element_release_pad<T: ElementImpl>(
     pad: *mut ffi::GstPad,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     // If we get a floating reference passed simply return here. It can't be stored inside this
@@ -488,7 +505,7 @@ unsafe extern "C" fn element_send_event<T: ElementImpl>(
     event: *mut ffi::GstEvent,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     panic_to_error!(&wrap, imp.panicked(), false, {
@@ -502,7 +519,7 @@ unsafe extern "C" fn element_query<T: ElementImpl>(
     query: *mut ffi::GstQuery,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
     let query = QueryRef::from_mut_ptr(query);
 
@@ -517,7 +534,7 @@ unsafe extern "C" fn element_set_context<T: ElementImpl>(
     context: *mut ffi::GstContext,
 ) {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     panic_to_error!(&wrap, imp.panicked(), (), {
@@ -530,7 +547,7 @@ unsafe extern "C" fn element_set_clock<T: ElementImpl>(
     clock: *mut ffi::GstClock,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     let clock = Option::<crate::Clock>::from_glib_borrow(clock);
@@ -545,7 +562,7 @@ unsafe extern "C" fn element_provide_clock<T: ElementImpl>(
     ptr: *mut ffi::GstElement,
 ) -> *mut ffi::GstClock {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     panic_to_error!(&wrap, imp.panicked(), None, {
@@ -559,7 +576,7 @@ unsafe extern "C" fn element_post_message<T: ElementImpl>(
     msg: *mut ffi::GstMessage,
 ) -> glib::ffi::gboolean {
     let instance = &*(ptr as *mut T::Instance);
-    let imp = instance.impl_();
+    let imp = instance.imp();
     let wrap: Borrowed<Element> = from_glib_borrow(ptr);
 
     // Can't catch panics here as posting the error message would cause
@@ -701,6 +718,8 @@ mod tests {
             }
         }
 
+        impl GstObjectImpl for TestElement {}
+
         impl ElementImpl for TestElement {
             fn metadata() -> Option<&'static ElementMetadata> {
                 use once_cell::sync::Lazy;
@@ -787,7 +806,7 @@ mod tests {
         let src = ElementFactory::make("fakesrc", None).unwrap();
         let sink = ElementFactory::make("fakesink", None).unwrap();
 
-        src.set_property("num-buffers", &100i32).unwrap();
+        src.set_property("num-buffers", 100i32);
 
         pipeline
             .add_many(&[&src, element.upcast_ref(), &sink])
@@ -802,7 +821,7 @@ mod tests {
 
         pipeline.set_state(crate::State::Null).unwrap();
 
-        let imp = imp::TestElement::from_instance(&element);
+        let imp = element.imp();
         assert_eq!(imp.n_buffers.load(atomic::Ordering::SeqCst), 100);
         assert!(imp.reached_playing.load(atomic::Ordering::SeqCst));
     }
