@@ -1,5 +1,7 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+#![allow(clippy::upper_case_acronyms)]
+
 use glib::translate::{from_glib, ToGlibPtr};
 use glib::{Date, SendValue, ToValue};
 
@@ -330,11 +332,11 @@ mod tests {
             tags.add::<Bitrate>(&96_000, TagMergeMode::Append); // u32
             tags.add::<TrackGain>(&1f64, TagMergeMode::Append); // f64
             tags.add::<Date>(
-                &glib::Date::new_dmy(28, glib::DateMonth::May, 2018).unwrap(),
+                &glib::Date::from_dmy(28, glib::DateMonth::May, 2018).unwrap(),
                 TagMergeMode::Append,
             );
             tags.add::<DateTime>(
-                &crate::DateTime::new_ymd(2018, 5, 28).unwrap(),
+                &crate::DateTime::from_ymd(2018, 5, 28).unwrap(),
                 TagMergeMode::Append,
             );
 
@@ -350,7 +352,7 @@ mod tests {
             tags.add::<Image>(&sample, TagMergeMode::Append); // Sample
         }
 
-        let pretty_config = ron::ser::PrettyConfig::new().with_new_line("".to_string());
+        let pretty_config = ron::ser::PrettyConfig::new().new_line("".to_string());
 
         let res = ron::ser::to_string_pretty(&tags, pretty_config);
         assert_eq!(
@@ -475,11 +477,11 @@ mod tests {
         assert!((tags.index::<TrackGain>(0).unwrap().get() - 1f64).abs() < std::f64::EPSILON);
         assert_eq!(
             tags.index::<Date>(0).unwrap().get(),
-            glib::Date::new_dmy(28, glib::DateMonth::May, 2018).unwrap()
+            glib::Date::from_dmy(28, glib::DateMonth::May, 2018).unwrap()
         );
         assert_eq!(
             tags.index::<DateTime>(0).unwrap().get(),
-            crate::DateTime::new_ymd(2018, 5, 28).unwrap()
+            crate::DateTime::from_ymd(2018, 5, 28).unwrap()
         );
         let sample = tags.index::<Image>(0).unwrap().get();
         let buffer = sample.buffer().unwrap();
@@ -511,11 +513,11 @@ mod tests {
         assert!((tags.index::<TrackGain>(0).unwrap().get() - 1f64).abs() < std::f64::EPSILON);
         assert_eq!(
             tags.index::<Date>(0).unwrap().get(),
-            glib::Date::new_dmy(28, glib::DateMonth::May, 2018).unwrap()
+            glib::Date::from_dmy(28, glib::DateMonth::May, 2018).unwrap()
         );
         assert_eq!(
             tags.index::<DateTime>(0).unwrap().get(),
-            crate::DateTime::new_ymd(2018, 5, 28).unwrap()
+            crate::DateTime::from_ymd(2018, 5, 28).unwrap()
         );
         let sample = tags.index::<Image>(0).unwrap().get();
         let buffer = sample.buffer().unwrap();
@@ -540,11 +542,11 @@ mod tests {
             tags.add::<Bitrate>(&96_000, TagMergeMode::Append); // u32
             tags.add::<TrackGain>(&1f64, TagMergeMode::Append); // f64
             tags.add::<Date>(
-                &glib::Date::new_dmy(28, glib::DateMonth::May, 2018).unwrap(),
+                &glib::Date::from_dmy(28, glib::DateMonth::May, 2018).unwrap(),
                 TagMergeMode::Append,
             );
             tags.add::<DateTime>(
-                &crate::DateTime::new_ymd(2018, 5, 28).unwrap(),
+                &crate::DateTime::from_ymd(2018, 5, 28).unwrap(),
                 TagMergeMode::Append,
             );
 
@@ -592,7 +594,7 @@ mod tests {
         );
         assert_eq!(
             tags.index::<DateTime>(0).unwrap().get(),
-            crate::DateTime::new_ymd(2018, 5, 28).unwrap()
+            crate::DateTime::from_ymd(2018, 5, 28).unwrap()
         );
         let sample = tags.index::<Image>(0).unwrap().get();
         let buffer = sample.buffer().unwrap();
